/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Huawei Technologies
 *
 */

#ifndef __LIST_H__
#define __LIST_H__

#ifndef NULL
#ifdef __cplusplus
#define NULL 0
#else
#define NULL ((void *)0)
#endif
#endif

struct list_node {
    struct list_node *prev;
    struct list_node *next;
};

static inline void init_list_node_null(struct list_node *n)
{
    n->prev = NULL;
    n->next = NULL;
}

static inline void init_list_node(struct list_node *n)
{
    n->prev = n;
    n->next = n;
}

static inline void list_add_node(struct list_node *h, struct list_node *n)
{
    n->next = h;
    n->prev = h->prev;
    h->prev->next = n;
    h->prev = n;
}

static inline void list_del_node(struct list_node *n)
{
    struct list_node *prev = n->prev;
    struct list_node *next = n->next;
    next->prev = prev;
    prev->next = next;
}

static inline void list_del_node_init(struct list_node *n)
{
    list_del_node(n);
    init_list_node(n);
}

static inline void list_del_node_null(struct list_node *n)
{
    if ((n->next) && (n->prev)) {
        list_del_node(n);
    }
    init_list_node_null(n);
}

static inline int list_is_null(const struct list_node *n)
{
    return (n->prev == NULL) && (n->next == NULL);
}

static inline int list_is_empty(const struct list_node *h)
{
    return h == h->next;
}

#define list_for_each_safe(pos, n, head) \
    for (pos = (head)->next, n = (pos)->next; pos != (head); pos = n, n = (pos)->next)

#ifndef container_of
#define container_of(ptr, type, member) ({ \
    typeof( ((type *)0)->member ) *__mptr = (ptr); \
    (type *)((char *)__mptr - offsetof(type,member));})
#endif /* container_of */

#endif /* __LIST_H__ */
